@@ -39,7 +39,7 @@ def matrix_symbols(names, dtype, rows, cols):
 
     matrices = []
     for n in names:
-        symbols = typed_symbols("%s:%i" % (n, rows * cols), dtype)
+        symbols = typed_symbols(f"{n}:{rows * cols}", dtype)
         matrices.append(sp.Matrix(rows, cols, lambda i, j: symbols[i * cols + j]))
 
     return tuple(matrices)
@@ -590,6 +590,28 @@ if int(sympy_version[0]) * 100 + int(sympy_version[1]) >= 109:
     sp.Number.__getstate__ = sp.Basic.__getstate__
     del sp.Basic.__getstate__
 
+    class FunctorWithStoredKwargs:
+        def __init__(self, func, **kwargs):
+            self.func = func
+            self.kwargs = kwargs
+
+        def __call__(self, *args):
+            return self.func(*args, **self.kwargs)
+
+    # __reduce_ex__ would strip kwargs, so we override it
+    def basic_reduce_ex(self, protocol):
+        if hasattr(self, '__getnewargs_ex__'):
+            args, kwargs = self.__getnewargs_ex__()
+        else:
+            args, kwargs = self.__getnewargs__(), {}
+        if hasattr(self, '__getstate__'):
+            state = self.__getstate__()
+        else:
+            state = None
+        return FunctorWithStoredKwargs(type(self), **kwargs), args, state
+    sp.Number.__reduce_ex__ = sp.Basic.__reduce_ex__
+    sp.Basic.__reduce_ex__ = basic_reduce_ex
+
 
 class Type(sp.Atom):
     def __new__(cls, *args, **kwargs):
@@ -612,10 +634,10 @@ class BasicType(Type):
             return 'ComplexDouble'
         elif name.startswith('int'):
             width = int(name[len("int"):])
-            return "int%d_t" % (width,)
+            return f"int{width}_t"
         elif name.startswith('uint'):
             width = int(name[len("uint"):])
-            return "uint%d_t" % (width,)
+            return f"uint{width}_t"
         elif name == 'bool':
             return 'bool'
         else:
@@ -714,7 +736,7 @@ class VectorType(Type):
 
     def __str__(self):
         if self.instruction_set is None:
-            return "%s[%s]" % (self.base_type, self.width)
+            return f"{self.base_type}[{self.width}]"
         else:
             if self.base_type == create_type("int64") or self.base_type == create_type("int32"):
                 return self.instruction_set['int']
